#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: log rotate and dump common include file
# File Name: esyslog_log_dump_common.sh
# Author: miaokaibo
# Create: 2022-04-15

source /opt/esyslog/esyslog_log_base_dump.sh

CUR_SCRIPT_NAME="${BASH_SOURCE}"

function fn_flock()
{
	local fd
	local -i ret=0
	local file="${ESYSLOG_LOCK_DIR}/$1"
	local -i wait_time=${2:-'-1'}
	local setted_umask=${3:-0077}
	local saved_umask
	local retry_num=3

	[ ! -d "${ESYSLOG_LOCK_DIR}" ] && mkdir -m 700 -p "${ESYSLOG_LOCK_DIR}"
	saved_umask=$(umask)
	umask "$setted_umask"
	eval "exec {fd}<>${file}"
	umask "$saved_umask"
	if [ ${wait_time} -ge 0 ] ; then
		flock -x -w ${wait_time} ${fd} >/dev/null 2>&1 # non-blocked
		ret=$?
	else
		while true
		do
			flock -x ${fd} >/dev/null 2>&1
			ret=$?
			if [ $ret -ne 0 ];then
				if [ "$retry_num" -gt 0 ];then
					retry_num=$((retry_num - 1))
					continue;
				fi
			fi
			break
		done
	fi
	if [ "$ret" -eq 0 ]; then
		echo "${fd}" >"${file}" # save fd for unlock
	else
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] fn_flock() locked ${file} failed, return value is ${ret}."
	fi
	return $ret
}

function fn_nonblock_flock()
{
	local file=$1
	fn_flock $file 0
}

function fn_unflock()
{
	local file="${ESYSLOG_LOCK_DIR}/$1"
	local fd=$(cat $file)
	eval "exec ${fd}<&-"
}

function fn_get_conf_section()
{
	local section="$1"
	local files="$2"
	local clean_confs
	local items
	local total_items=""
	local repeated_items

	for file in $files; do
		[ ! -f "$file" ] && continue;
		clean_confs=$(sed -e "/^[[:blank:]]*#/d; /^$/d" $file)
		items=$(printf "$clean_confs" | sed -n "/^\[$section\]$/,/^\[/p")
		items=$(printf "$items" | sed "/^\[/d")
		total_items="${total_items}${items}\n"
	done
	total_items=$(printf "$total_items" | sed "/^$/d" | sort)
	repeated_items=$(printf "$total_items" | uniq -d | tr "\n" " ")
	if [ -n "$repeated_items" ]; then
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] fn_get_conf_section() got duplicated content \"${repeated_items}\" in section ${section}."
	fi
	total_items=$(printf "$total_items" | uniq | tr "\n" " ")
	echo "$total_items"
}

fn_get_values_from_oslogdump_conf()
{
	fn_flock oslogdump.lock

	# Config field 'LOG_DUMP_LIMIT' is the size threhold to triger compressing temporary
	# directory for log collection - /var/log/logdump. (Unit: MB, should > 0)
	LOG_DUMP_LIMIT=`grep "^LOG_DUMP_LIMIT" $OS_LOGDUMP_CONF | awk -F "=" '{print $2}'`
	LOG_DUMP_LIMIT=`echo ${LOG_DUMP_LIMIT} | sed s/^0*//g`
	if ! (echo "${LOG_DUMP_LIMIT}" | grep -q "^[[:digit:]]\{1,\}$") ;then
		LOG_DUMP_LIMIT=100
	fi

	# Config field 'LOG_BAK_DIR' is the directory storing final dumped logs in every day
	# collection. It need be specified as absolute path.
	LOG_BAK_DIR=`grep "^LOG_BAK_DIR=" $OS_LOGDUMP_CONF | awk -F "=" '{print $2}'`

	if [ x"$LOG_BAK_DIR" = x ] || ! (echo $LOG_BAK_DIR | grep -q ^"/") ;then
		LOG_BAK_DIR=/var/log
	fi

	# Config field 'LOG_BAK_DIR_MAX_USAGE' is upper bound of disk usage which the LOG_BAK_DIR
	# mounted on. If the threhold is exceeded, we start to delete old logs to lower it down.
	LOG_BAK_DIR_MAX_USAGE=`grep "^LOG_BAK_DIR_MAX_USAGE" $OS_LOGDUMP_CONF | awk -F "=" '{print $2}' | sed -n "/^[0-9]\+$/p"`
	LOG_BAK_DIR_MAX_USAGE=`echo "${LOG_BAK_DIR_MAX_USAGE}" | sed s/^0*//g`
	if ! (echo "${LOG_BAK_DIR_MAX_USAGE}" | grep -q "^[[:digit:]]\{1,\}$") || [ $LOG_BAK_DIR_MAX_USAGE -lt 1 ] || [ $LOG_BAK_DIR_MAX_USAGE -ge 100 ] ;then
		LOG_BAK_DIR_MAX_USAGE=90
	fi

	# Config filed 'LOG_BAK_DIR_RESUME_USAGE' is the lower bound of disk usage. We continue
	# deleting logs in LOG_BAK_DIR until disk usage is lower than this threhold.
	LOG_BAK_DIR_RESUME_USAGE=`grep "^LOG_BAK_DIR_RESUME_USAGE" $OS_LOGDUMP_CONF | awk -F "=" '{print $2}' | sed -n "/^[0-9]\+$/p"`
	LOG_BAK_DIR_RESUME_USAGE=`echo "${LOG_BAK_DIR_RESUME_USAGE}" | sed s/^0*//g`
	if ! (echo "$LOG_BAK_DIR_RESUME_USAGE" | grep -q "^[[:digit:]\{1,\}$]" ) || [ $LOG_BAK_DIR_RESUME_USAGE -lt 1 ] || [ $LOG_BAK_DIR_RESUME_USAGE -ge 100 ] ;then
		LOG_BAK_DIR_RESUME_USAGE=80
	fi
	if [ ${LOG_BAK_DIR_RESUME_USAGE} -gt ${LOG_BAK_DIR_MAX_USAGE} ];then
		LOG_BAK_DIR_RESUME_USAGE=${LOG_BAK_DIR_MAX_USAGE}
	fi

	LOG_SAVE_DAYS=`grep "^CONFIG_SAVE_DAYS" ${OS_LOGDUMP_CONF} | awk -F "=" '{print $2}'`

	LOG_SAVE_SIZE=`grep "^CONFIG_SAVE_SIZE" ${OS_LOGDUMP_CONF} | awk -F "=" '{print $2}' | sed -n "/^[0-9]\+$/p"`
        if ! (echo "$LOG_SAVE_SIZE" | grep -q "^[[:digit:]\{1,\}$]" ) ;then
		LOG_SAVE_SIZE=500
	fi

	fn_unflock oslogdump.lock
}


############################################################################
# Function Name: fn_chk_log_partition
# Description: check log partition over maxmum limit
# Parameter: 1:destination directories that the source file save to
#            2:destination directories that maximum use percentage
#            3:destination directories that resume percentage
###########################################################################
function fn_chk_log_partition()
{
	local logbakdir=$1
	local log_bak_dir_max_usage=$2
	local log_bak_dir_resume_usage=$3
	local partition_use=`df ${logbakdir} | sed -n '2p' | awk '{print $5}' | sed 's/%//g'`

	# if over logbakdir_MAX_USAGE, delete wtmp btmp vm-*.con and the oldest log file util under resume usage
	if [ ${partition_use} -ge ${log_bak_dir_max_usage} ];then
		${G_LOG_PRINT_CMD} "notice" "[${CUR_SCRIPT_NAME}] ${logbakdir} in Disk space use is more than ${log_bak_dir_max_usage}%, removing logs"

		while [ ${partition_use} -ge $log_bak_dir_resume_usage ]
		do
			local del_file=`ls ${logbakdir} | grep "^logdump-.*-.*\.tar\.gz$" | sort -t- -k2n | cut -d '-' -f2,3 | head -n1`
			#varlog_history only exist when /var/log dir in ram disk
			local del_varlog_history=`ls ${logbakdir} | grep "^varlog_history-.*-.*\.tar\.gz$" | sort -t- -k2n | cut -d '-' -f2,3 | head -n1`

			if [[ x${del_file} == "x" ]] && [[ x${del_varlog_history} == "x" ]] ;then
				partition_use=`df ${logbakdir} | sed -n '2p' | awk '{print $5}' | sed 's/%//g'`
				if [ ${partition_use} -ge $log_bak_dir_max_usage ];then
					${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error: Disk space is not enough for log dumping."
					return 1
				fi
				break
			fi

			if [ ! -z ${del_file} ];then
				rm -f "${logbakdir}/logdump-${del_file}"
			fi

			if [ ! -z ${del_varlog_history} ];then
				rm -f "${logbakdir}/varlog_history-${del_varlog_history}"
			fi

			partition_use=`df ${logbakdir} | sed -n '2p' | awk '{print $5}' | sed 's/%//g'`
		done
	fi
	return 0
}

function fn_tar_logdump_dir()
{
	local logdumpdir=$2
	local max_id=0
	local bakdir=$1
	max_id=`ls ${bakdir} | grep "logdump-.*-.*\.tar\.gz$" | cut -d '-' -f2 | sort -nr | head -n1`
	if [ -z ${max_id} ];then
		max_id=0
	fi
	((new_id=${max_id}+1))
	file_name="logdump-${new_id}-`date "+%Y%m%d%H%M%S"`.tar.gz"
	pushd ${logdumpdir} > /dev/null
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] Warn: pushd ${logdumpdir} failed"
		return 1
	fi

	tar -zcf ${bakdir}/${file_name} -- * 2>&1 | logger -it esyslog_log_dump_common.sh -p err
	if [ ${PIPESTATUS[0]} -ne 0 ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] command [tar -zcf ${bakdir}/${file_name} .] failed"
	fi
	chmod 400 ${bakdir}/${file_name}
	popd > /dev/null
	rm "${logdumpdir}" -rf 2>&1 | logger -it esyslog_log_dump_common.sh -p warning
}

################################################################################
# Function Name: fn_dump_logfile
# Description: rotate the source log, and dump the file to LOG_BAK_DIR
# Parameter:  destination directories that the source file save to
################################################################################
function fn_dump_logfile()
{
	local logbakdir=$1

	if [ ! -d "${logbakdir}" ];then
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] logbakdir:${logbakdir} diretory is not exit."
		return 1
	fi

	#处理已经mv但是未打包的logdump.1包
	if [ -d "${logbakdir}/logdump.1" ];then
		fn_tar_logdump_dir ${logbakdir} ${logbakdir}/logdump.1
		if [ $? -ne 0 ];then
			${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] Warn: fn_tar_logdump_dir ${logbakdir} ${logbakdir}/logdump.1 failed"
			return 1
		fi
	fi

	mv ${logbakdir}/logdump ${logbakdir}/logdump.1 >/dev/null 2>&1
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}] Warn: mv ${logbakdir}/logdump to ${logbakdir}/logdump.1 failed"
		return 1
	fi
	fn_tar_logdump_dir ${logbakdir} ${logbakdir}/logdump.1
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "warning" "[${CUR_SCRIPT_NAME}]Warn: fn_tar_logdump_dir ${logbakdir} ${logbakdir}/logdump.1 failed"
		return 1
	fi
	return 0
}

################################################################################
# Function Name: dir_is_var_log
# Description: Check that LOG_BAK_DIR is the /var/log directory
# Parameter:  LOG_BAK_DIR dir
# Return_value: 0->LOG_BAK_DIR!=/var/log;1->LOG_BAK_DIR==/var/log
################################################################################
function dir_is_var_log()
{
	local log_bak_dir="$1"

	[ -d "${log_bak_dir}" ] || mkdir -p "${log_bak_dir}" -m 700 2>&1 || logger -it esyslog_log_dump_common "create ${log_bak_dir} fail" -p warning
	if [ "$(/usr/bin/realpath $log_bak_dir 2>/dev/null)" != "/var/log" ];then
		return 0
	else
		return 1
	fi
}

################################################################################
# Function Name: fn_rotatelog
# Description: mv the log to logdump dir
# Parameter:  1:destination directories that the source file save to
#             2:logdump config files
################################################################################
function fn_rotatelog()
{
	local dump_files=""
	local rotate_file=""
	local dump_dir=""
	local tmp_logdump_dir="${LOG_BAK_DIR}"
	local dest_bakdir=""
	local logdump_log_files="${1}"
	local ret=0

	if [ $diskflag -eq 1 ];then
		tmp_logdump_dir="${LOG_DIR}"
		if [ -f "${LOG_BAK_DIR}/logdump.1" ];then
			 rm "${LOG_BAK_DIR}"/logdump.1 -f 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
			 [ ${PIPESTATUS[0]} -ne 0 ] && return 1
		fi
		if [ ! -d "${LOG_BAK_DIR}/logdump" ];then
			if [ -f "${LOG_BAK_DIR}/logdump" ];then
				rm "${LOG_BAK_DIR}"/logdump -f 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
				[ ${PIPESTATUS[0]} -ne 0 ] && return 1
			fi
			mkdir -p ${LOG_BAK_DIR}/logdump -m 700 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
			[ ${PIPESTATUS[0]} -ne 0 ] && return 1
                fi
	fi
	if [ -f "${tmp_logdump_dir}/logdump.1" ];then
		rm "${tmp_logdump_dir}"/logdump.1 -f 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
		[ ${PIPESTATUS[0]} -ne 0 ] && return 1
	fi
	if [ ! -d "${tmp_logdump_dir}/logdump" ];then
		if [ -f "${tmp_logdump_dir}/logdump" ];then
			rm "${tmp_logdump_dir}"/logdump -f 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
			[ ${PIPESTATUS[0]} -ne 0 ] && return 1
		fi
		mkdir -p ${tmp_logdump_dir}/logdump -m 700 2>&1 | logger -it "esyslog_log_dump_common.sh" -p warning
		[ ${PIPESTATUS[0]} -ne 0 ] && return 1
	fi

	for line in ${logdump_log_files}
	do
		if [ -d "${line}" ];then
			dump_dir=${line/${LOG_DIR}/${tmp_logdump_dir}\/logdump}

			if [ ! -d "${dump_dir}" ];then
				mkdir -p ${dump_dir} -m 700
			fi
			dump_files=$(ls ${line} 2>/dev/null)
			for onefile in ${dump_files}
			do
				if [ -f "${line}/${onefile}" ];then
					rotate_file=`echo "${line}/${onefile}" | grep ".*.[0-9]\{1,\}$"`
					if [ ! -z "${rotate_file}" ];then
						date=`date "+%Y%m%d%H%M%S"`
						file_name=${onefile}-${date}
						exist_file=`ls ${dump_dir} | grep ${file_name}`
						if [ ! -z "${exist_file}" ];then
							max_rotate_id=`ls ${dump_dir} | grep "${onefile%.*}\..*-${date}" | awk -F "." '{print $NF}'  | awk -F "-" '{print $1}' | sort -nr | head -n1`
							((new_id=${max_rotate_id}+1))
							file_name=${file_name%.*}.${new_id}-${date}
						fi
						mv ${line}/${onefile} ${dump_dir}/${file_name}
						chmod 400 ${dump_dir}/${file_name}
						# 0->LOG_BAK_DIR!=/var/log; 1->LOG_BAK_DIR=/var/log
						dir_is_var_log "${LOG_BAK_DIR}"
						ret="$?"
						# Only if /var/log is in memory and LOG_BAK_DIR!=/var/log, Logs will be synchronized to disk.
						if [ ${diskflag} -eq 1 ] && [ ${ret} -eq 0 ];then
							dest_bakdir=${line/${LOG_DIR}/${LOG_BAK_DIR}\/logdump\/}
							${ESYSLOG_LOG_SAVE_TO_DISK} "${dump_dir}/${file_name}" "${dest_bakdir}" "${LOG_BAK_DIR}" "${LOG_DUMP_LIMIT}" &
						fi

					fi
					# delete monitor file by imfile
					rm -f "${line}/${onefile}".tmp
				fi
			done
		elif [ -f "${line}" ] || [ -f "${line}.1" ];then
			line_dir=${line%/*}
			line_file_name="${line##*/}"
			dump_dir=${line_dir/${LOG_DIR}/${tmp_logdump_dir}\/logdump}
			if [ ! -d "${dump_dir}" ];then
				mkdir -p ${dump_dir} -m 700
			fi

			dump_files=$(ls ${line}.* 2>/dev/null)
			for onefile in ${dump_files}
			do
				rotate_file=`echo "${onefile}" | grep ".*.[0-9]\{1,\}"`
				if [ -z "${rotate_file}" ];then
					continue
				fi
				date=`date "+%Y%m%d%H%M%S"`
				file_name=${onefile}-${date}
				exist_file=`ls ${dump_dir} | grep ${file_name##*/}`
				if [ ! -z "${exist_file}" ];then
					max_rotate_id=`ls ${dump_dir} | grep "${line_file_name}\..*-${date}" | awk -F "." '{print $NF}'  | awk -F "-" '{print $1}' | sort -nr | head -n1`
					((new_id=${max_rotate_id}+1))
					file_name=${line}.${new_id}-${date}
				fi
				mv ${onefile} ${dump_dir}/${file_name##*/}
				chmod 400 ${dump_dir}/${file_name##*/}
				# 0->LOG_BAK_DIR!=/var/log; 1->LOG_BAK_DIR=/var/log
				dir_is_var_log "${LOG_BAK_DIR}"
				ret="$?"
				# Only if /var/log is in memory and LOG_BAK_DIR(oslogdump.conf)!=/var/log, Logs will be synchronized to disk
				if [ ${diskflag} -eq 1 ] && [ ${ret} -eq 0 ];then
					dest_bakdir=${line_dir/${LOG_DIR}/${LOG_BAK_DIR}\/logdump\/}
					${ESYSLOG_LOG_SAVE_TO_DISK} "${dump_dir}/${file_name##*/}" "${dest_bakdir}" "${LOG_BAK_DIR}" "${LOG_DUMP_LIMIT}" &
				fi
				done
			# delete monitor file by imfile
			rm -f "${line}".tmp
		fi
	done
	logger -it "esyslog_log_dump_common.sh" -p notice \
		"fn_rotatelog processed log files/dirs: \"${logdump_log_files}\""
	return 0
}
