#!/bin/bash
# Copyright (c) 2022 xFusion Digital Technologies Co., Ltd. All rights reserved.
# Description: save the source file to destination directories in the disk.
# File Name: esyslog_log_save_to_disk.sh
# Author: miaokaibo
# Create: 2022-04-15


source /opt/esyslog/esyslog_log_dump_common.sh

CUR_SCRIPT_NAME="${BASH_SOURCE}"

fn_free_logdumpdir_size()
{
	local filename=$1
	local src_dir=$2
	local dest_dir=$3
	local logbakdir=$4
	local filesize=0
	local disk_available_size=0

	filesize=`du -sk ${src_dir}/${filename} |awk '{print $1}'`
	disk_available_size=`df -lk ${dest_dir} | sed -n '2p' | awk '{print $4}'`
	if [ $filesize -lt $disk_available_size ];then
		return 0
	fi
	while [ $filesize -ge $disk_available_size ]
	do
		old_file=`ls ${logbakdir} | grep "logdump-.*-.*\.tar\.gz" | cut -d '-' -f3 | sort -n | head -n1`
		if [ -n "${old_file}" ];then
			del_file=`ls ${logbakdir} | grep "logdump-.*-$old_file"`
			rm -f "${logbakdir}/${del_file}"
		else
			${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, there is not enough space for save ${filename} to ${dest_dir}."
			return 1
		fi
		disk_available_size=`df ${dest_dir} | sed -n '2p' | awk '{print $4}'`
	done
	return 0

}

fn_save_file_to_disk()
{
	local filename=$1
	local src_dir=$2
	local dest_dir=$3
	local logbakdir=$4
	local logname=${filename%\.*}
	local rotatetime=""
	local destfilename=$1

	if [ -f "${dest_dir}/${filename}" ];then
		rotatetime=${filename##*-}
		max_rotate_id=`ls ${dest_dir} | grep "${logname}\..*-${rotatetime}" | awk -F "." '{print $NF}'  | awk -F "-" '{print $1}' | sort -nr | head -n1`
		((new_id=${max_rotate_id}+1))
		destfilename=${logname}.${new_id}-${rotatetime}
	fi

	# Check if disk usage which target directory mounted on is full.
	# If so, just delete old logdump tar files.
	fn_free_logdumpdir_size "$filename" "$src_dir" "$dest_dir" "$logbakdir"
	cp ${src_dir}/${filename} "${dest_dir}/${destfilename}" >/dev/null 2>&1
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, cp ${src_dir}/${filename} to ${dest_dir}/${filename} failed , rm -f ${src_dir}/${filename} ."
	else
		which fsync &>/dev/null
		if [ $? -ne 0 ];then
			which sync &>/dev/null && sync
		else
			fsync ${dest_dir}/${destfilename}
		fi
	fi
	rm -f "${src_dir}/${filename}"

}

################################################################################
# Function Name: fn_save_file_to_disk
# Description: save source file to destination dir.
# Parameter: parameter one, the file which needs to be saved.
#            parameter two, the destination directory for saving the file.
#            parameter three, the flag means log bak dir.
#            parameter four, logdump dir limit size
# output：none
# Return: 0-save to dir successfully, 1-save to dir failed
################################################################################

main()
{
	fn_flock logdump_to_disk.lock
	local src_file="$1"
	local dest_dir="$2"
	local logbakdir="$3"
	local log_dump_limit="$4"
	local filename=""
	local tmpbakfile=""
	local cur_size=0

	if [ -z "${src_file}" ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, src_file[${src_file}] is null."
		return 1
	else
		if [ ! -f "$src_file" ];then
			${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, src_file[${1}] is not exist."
			return 1
		fi
	fi

	if [ -z "${dest_dir}" ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, dest_dir[${dest_dir}] is null."
		return 1
	else
		if [ ! -d "${dest_dir}" ];then
			mkdir -p "${dest_dir}" -m 700
		fi
	fi

	if [ -z "${logbakdir}" ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, logbakdir[${logbakdir}] is null."
		return 1
        fi

	if [ -z "$(echo $log_dump_limit | grep '^[[:digit:]]*$')" ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, log_dump_limit[${log_dump_limit}] is abnormal parameter."
		return 1
	fi

	filename=${src_file##*/}
	logname=${filename%\.*}
	#check for unprocessed filename
	for file in `ls ${src_file%/*}/ | grep $logname | grep -v $filename`
	do
		if ! (ps -efww | grep -v grep | grep "${ESYSLOG_LOG_SAVE_TO_DISK}" | grep -q "$file") ;then

			fn_save_file_to_disk "$file" "${src_file%/*}" "$dest_dir" "$logbakdir"
		fi
	done

	fn_save_file_to_disk "$filename" "${src_file%/*}" "$dest_dir" "$logbakdir"
	if [ $? -ne 0 ];then
		${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] Error, fn_save_file_to_disk ${filename} ${src_file%/*} ${dest_dir} ${logbakdir}."
	fi
	# dump the log file to ${logbakdir}
	if [ -d ${logbakdir}/logdump ];then
		cur_size=`du -sm "${logbakdir}/logdump" | awk '{print $1}'`
	fi
	if [ "${cur_size}" -gt "${log_dump_limit}" ];then
		fn_dump_logfile ${logbakdir}
		if [ $? -ne 0 ];then
			${G_LOG_PRINT_CMD} "err" "[${CUR_SCRIPT_NAME}] fn_dump_logfile ${logbakdir} failed."
			return 1
		fi
	fi
	fn_unflock logdump_to_disk.lock
	return 0
}

main "$@"
exit $?
